#The following R code was written for the goodness-of-fit using test statistic based on Claeskens and Hart (2009, TEST) for the skewed normal:

#Model: yi=b1*xi+b2*x^{2}_i+vi+ei where v_i~SN(0,A,alpha) and ei~N(0,Di)
#where yi:sample survey of 4-person median income in yr 1989.
#xi:census 4-person median income in yr 1979. 

rm(list=ls(all=TRUE))

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(nlme)
library(MASS)
library(psych)
library(sn)
#----------------------------------------
#Program:
# square-root-inverse of a matrix
mat.sqrt.inv<-function(A)
{
  ei<-eigen(A)
  d<-ei$values
  d<-(d+abs(d))/2
  d2<-1 / sqrt(d)
  d2[d == 0]<-0
  ans<-ei$vectors %*% diag(d2) %*% t(ei$vectors)
  return(ans)
}
#=============================================
#Define the constants of the data and model

m=51
p1=2 #number of regression coefficents: beta1, beta2
p=3 #number of model parameters: beta, A
d=1  #gamma parameter
q=p+d

n1=m/2

Median<-read.table("fmedn89-1.txt",header=TRUE)
attach(Median)

yib<-rep(0,m)
D<-rep(0,m)
xi<-rep(0,m)

bb<-0
for(i in 1:m){
  yib[i]<-X4Med[13+bb]
  D[i]<-(X4_SE[13+bb])^2
  bb<-bb+13
}
yib
D
SD<-sqrt(D)

dd<-0
for(i in 1:m){xi[i]<-X4Med[2+dd]
dd<-dd+13}
xi<-xi/1000 #we devide xi by 1000 for the ease of computation 

X<-cbind(xi,xi^2)
   ######################################################################################
    ############################################M=0########################################
    #######################################################################################
    
    log.lik.0<-function(THETA){
      BETA1<-THETA[1:2]
      A<-THETA[3]
      
      cum0<-0
      for(i in 1:m){
        cum0<-cum0 +log( dnorm(yib[i],mean=X[i,]%*%BETA1, sd=sqrt(A+D[i])) ) 
      }
      return(cum0) 
    }
    #-----------
    #log-likelihood for M=0
    THETA<-c(2,0.1,18818686)
    fit.0 <- optim(THETA, log.lik.0, method="BFGS", hessian = TRUE, control=list(fnscale=-1, maxit=1000))
    
    print(paste("------------ Estimation of model parameters for M=0"))
    print(fit.0$par)
    
    print(paste("------------ Convergence for M=0"))
    print(fit.0$convergence)
    
    print(paste("------------ Log-likelihood for M=0"))
    print(fit.0$value)
    
    #######################################################################################
    ############################################M=1########################################
    #######################################################################################
    log.lik.1<-function(THETA){
      BETA1<-THETA[1:2]
      PHI<-THETA[3]
      A<-THETA[4]
      
      cum0<-0
      for(i in 1:m){
        cum0<-cum0+log( dnorm(yib[i],mean=X[i,]%*%BETA1, sd=sqrt(A+D[i])) )
      }
      
      cum1<-0
      c<-rep(0,2)
      c[1]<-sinpi(PHI)
      c[2]<-cospi(PHI)
      
      for(i in 1:m){ 
        gamma<-A/(A+D[i])
        Ai<-matrix(c(1,gamma*(yib[i]-X[i,]%*%BETA1), gamma*(yib[i]-X[i,]%*%BETA1), 
                     A*(1-gamma)+ gamma^2*(yib[i]-X[i,]%*%BETA1)^2),2,2,byrow=T)
        Bi<-mat.sqrt.inv(Ai)
        BETA2<-Bi%*%c
        cum1<-cum1+log( BETA2[1]^2+BETA2[2]^2*( A*(1-gamma)+gamma^2*(yib[i]-X[i,]%*%BETA1)^2  )+
                          2*(BETA2[1]*BETA2[2])*gamma*(yib[i]-X[i,]%*%BETA1) )
      }
      return(cum0+cum1) 
    }
    #------------------
    #log-likelihood for M=1
    THETA<-c(2,0,1/3,18818686)
    fit.1 <- optim(THETA, log.lik.1, method="BFGS", hessian = TRUE, control=list(fnscale=-1, maxit=1000))
    
    TOS1<-2*(fit.1$value-fit.0$value)/(1*(1+3)/2)
    
    print(paste("------------ Estimation of model parameters for M=1"))
    print(fit.1$par)
    
    print(paste("------------ Convergence for M=1"))
    print(fit.1$convergence)
    
    print(paste("------------ Log-likelihood for M=1"))
    print(fit.1$value)
    #-------------------
    TOS.tem<-(TOS1)
    
    print(paste("------------ Test statistic under H0"))
    print(TOS.tem)
    #--------------------
    B1=100
    TOS.1<-rep(0,B1)
    
    for(b1 in 1:B1){  #loop for B1 to obtain the critical value (using the bootstrap-like approach)
      
      random1<-rnorm(m,0,1)
      random2<-rnorm(m,0,1)
      yib.1<-rep(0,m)
      for(i in 1:n1){
        yib.1[i]<-(fit.0$par[1])*xi[i]+(fit.0$par[2])*xi[i]^2+sqrt(fit.0$par[3])*random1[i]+SD[i]*random2[i]
      }
      for(i in (n1+1):m){
        yib.1[i]<-(fit.0$par[1])*xi[i]+(fit.0$par[2])*xi[i]^2+sqrt(fit.0$par[3])*random1[i]+SD[i]*random2[i]
      }
      
      #######################################################################################
      ############################################M=0########################################
      #######################################################################################
      
      log.lik.0.1<-function(THETA){
        BETA1<-THETA[1:2]
        A<-THETA[3]
        
        cum0<-0
        for(i in 1:m){
          cum0<-cum0 +log( dnorm(yib.1[i],mean=X[i,]%*%BETA1, sd=sqrt(A+D[i])) ) 
          # OR cum0<-cum0+log(  (1/(sqrt(2*pi*(A+D[i])))) *exp(-(yib.1[i]-BETA1*xi[i])^2/(2*(A+D[i]))) )
        }
        return(cum0) 
      }
      #-----------
      #log-likelihood for M=0
      THETA<-c(2,0, 18818686)
      fit.0.1 <- optim(THETA, log.lik.0.1, method="BFGS", hessian = TRUE, control=list(fnscale=-1, maxit=1000))
      
      
      Est.0.1<-fit.0.1$par
      logl0.1<-fit.0.1$value
      conv0.1<-fit.0.1$convergence
      
      #######################################################################################
      ############################################M=1########################################
      #######################################################################################
      
      log.lik.1.1<-function(THETA){
        BETA1<-THETA[1:2]
        PHI<-THETA[3]
        A<-THETA[4]
        
        cum0<-0
        for(i in 1:m){
          cum0<-cum0+log( dnorm(yib.1[i],mean=X[i,]%*%BETA1, sd=sqrt(A+D[i])) )
        }
        
        cum1<-0
        c<-rep(0,2)
        c[1]<-sinpi(PHI)
        c[2]<-cospi(PHI)
        
        for(i in 1:m){ 
          gamma<-A/(A+D[i])
          Ai<-matrix(c(1,gamma*(yib.1[i]-X[i,]%*%BETA1), gamma*(yib.1[i]-X[i,]%*%BETA1), 
                       A*(1-gamma)+ gamma^2*(yib.1[i]-X[i,]%*%BETA1)^2),2,2,byrow=T)
          Bi<-mat.sqrt.inv(Ai)
          BETA2<-Bi%*%c
          cum1<-cum1+log( BETA2[1]^2+BETA2[2]^2*( A*(1-gamma)+gamma^2*(yib.1[i]-X[i,]%*%BETA1)^2  )+
                            2*(BETA2[1]*BETA2[2])*gamma*(yib.1[i]-X[i,]%*%BETA1) )
        }
        return(cum0+cum1) 
      }
      #------------------
      #log-likelihood for M=1
      THETA<-c(2,0,1/3,18818686)
      fit.1.1 <- optim(THETA, log.lik.1.1, method="BFGS", hessian = TRUE, control=list(fnscale=-1, maxit=1000))
      
      
      Est.1.1<-fit.1.1$par
      logl1.1<-fit.1.1$value
      conv1.1<-fit.1.1$convergence
      
      TOS.1.1<-2*(logl1.1-logl0.1)/ (1*(1+3)/2)
      
      TOS.1[b1]<-(TOS.1.1)
      cat(b1)
    }#loop for B1.
    
    
    print(paste("------------ Critical value under H0 "))
    print(quantile(TOS.1, probs =(0.95)))
    #95% 
    #6.838263e-12
    
    TOS.tem
   #[1] 1.705303e-13 
    #Since 1.705303e-13 < 6.838263e-12, the H0 is not rejected. It means that 
    #the distribution of vi can be assumed normal rather than the skewed normal. 
########################################################################    