#Goodness-of-fit using test statistic based on chi-square result of Jinag (2001) for the skewed normal:

#Model: yi=b1*xi+b2*x^{2}_i+vi+ei where v_i~SN(0,A,alpha) and ei~N(0,Di)
#where yi:sample survey of 4-person median income in yr 1989.
#xi:census 4-person median income in yr 1979. 


rm(list=ls(all=TRUE))

#---------------
runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(nlme)
library(MASS)
library(psych)
library(sn)

m=51  

p=2 #number of regression coefficents: beta1, beta2
M=round(max(p+2,m^(0.2)))
Nk=round(m/M)

Median<-read.table("fmedn89-1.txt",header=TRUE)
attach(Median)

yib<-rep(0,m)
D<-rep(0,m)
xi<-rep(0,m)

bb<-0
for(i in 1:m){
  yib[i]<-X4Med[13+bb]
  D[i]<-(X4_SE[13+bb])^2
  bb<-bb+13
}
yib
D
SD<-sqrt(D)

dd<-0
for(i in 1:m){xi[i]<-X4Med[2+dd]
dd<-dd+13}
xi

X<-cbind(xi,xi^2)
###################################
PX<-X%*% solve(t(X)%*%X)%*%t(X)
PXI<-diag(1,m)-PX
D1<-diag(SD^2)
  #####We use the the PR and EBLUE##############
  A.hat.pr<-(t(yib)%*%PXI%*%yib-tr(PXI%*%D1))/(m-p)
  A.hat.pr
 # [1,] 18818686
  
  sumx.tem<-matrix(0,p,p)
  sumy.tem<-matrix(0,p,1)
  for(i in 1:m){
    sumx.tem<-sumx.tem+( X[i,]%*%t(X[i,] ) ) /drop(A.hat.pr+SD[i]^2) 
    sumy.tem<-sumy.tem+( X[i,]*yib[i] )/drop(A.hat.pr+SD[i]^2)
  }
  BETA.hat<-solve(sumx.tem)%*%sumy.tem
  BETA.hat
  #xi  2.0686457882
  #-0.0000126125
  ############################################
  ysort<-sort(yib)
  c1=ysort[round(m/4)]
  c2=ysort[round(m/2)]
  c3=ysort[round(3*m/4)]
  C<-c(c1,c2,c3)
  Phi<-matrix(0,M,m)
  
  for(i in 1:m){
    tem<-(C[1]-X[i,]%*%BETA.hat)/sqrt(A.hat.pr+SD[i])
    Phi[1,i]<-pnorm(tem, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
  }
  
  for(k in 2:(M-1)){
    for(i in 1:m){
      temk<-(C[k]-X[i,]%*%BETA.hat)/sqrt(A.hat.pr+SD[i])
      temk1<-(C[k-1]-X[i,]%*%BETA.hat)/sqrt(A.hat.pr+SD[i])
      Phi[k,i]<-pnorm(temk, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)-pnorm(temk1, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
    }
  }
  
  
  for(i in 1:m){
    tem<-(C[M-1]-X[i,]%*%BETA.hat)/sqrt(A.hat.pr+SD[i])
    Phi[M,i]<-1-pnorm(tem, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
  }
  
  P<-rep(0,M)
  for(k in 1:M){
    for(i in 1:m){
      P[k]<-P[k]+Phi[k,i]
    }
  }
  
  ChiSq.bp<-(1/m)*sum((Nk-P)^2)
  ##################Bootstrap loop#######################
  B1=1000
  ChiSq.bp.boot<-rep(0,B1)
  for(b1 in 1:B1){
    
    random1<-rnorm(m,0,1)
    random2<-rnorm(m,0,1)
    yib.b<-rep(0,m)
    for(i in 1:m){
      yib.b[i]<-BETA.hat[1]*xi[i]+BETA.hat[2]*xi[i]^2+sqrt(A.hat.pr)*random1[i]+SD[i]*random2[i]
    }
    
    A.hat.boot<-(t(yib)%*%PXI%*%yib.b-tr(PXI%*%D1))/(m-p)
    A.hat.boot<-ifelse(A.hat.boot<=0,0.0001,A.hat.boot)
    
    sumx<-matrix(0,p,p)
    sumy<-matrix(0,p,1)
    for(i in 1:m){
      sumx<-sumx+( X[i,]%*%t(X[i,] ) ) /drop(A.hat.boot+SD[i]^2) 
      sumy<-sumy+( X[i,]*yib.b[i] )/drop(A.hat.boot+SD[i]^2)
    }
    BETA.hat.boot<-solve(sumx)%*%sumy
    
    ysort<-sort(yib.b)
    c1=ysort[round(m/4)]
    c2=ysort[round(m/2)]
    c3=ysort[round(3*m/4)]
    C<-c(c1,c2,c3)
    Phi<-matrix(0,M,m)
    
    for(i in 1:m){
      tem<-(C[1]-X[i,]%*%BETA.hat.boot)/sqrt(A.hat.boot+SD[i])
      Phi[1,i]<-pnorm(tem, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
    }
    
    for(k in 2:(M-1)){
      for(i in 1:m){
        temk<-(C[k]-X[i,]%*%BETA.hat.boot)/sqrt(A.hat.boot+SD[i])
        temk1<-(C[k-1]-X[i,]%*%BETA.hat.boot)/sqrt(A.hat.boot+SD[i])
        Phi[k,i]<-pnorm(temk, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)-pnorm(temk1, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
      }
    }
    
    
    for(i in 1:m){
      tem<-(C[M-1]-X[i,]%*%BETA.hat.boot)/sqrt(A.hat.boot+SD[i])
      Phi[M,i]<-1-pnorm(tem, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)
    }
    
    P<-rep(0,M)
    for(k in 1:M){
      for(i in 1:m){
        P[k]<-P[k]+Phi[k,i]
      }
    }
    
    
    ChiSq.bp.boot[b1]<-(1/m)*sum((Nk-P)^2)
  }#loop of B1
  #======================
  tem<-quantile(ChiSq.bp.boot, probs = c(0.95))
  tem
  #95% 
  #10.94353 
  ChiSq.bp
  #[1] 0.7737698
  #since the test statistic=0.77 is much smaller then the treshhold value 10.94, the H0 is not rejected. It means that 
  #the distribution of vi can be assumed normal rather than the skewed normal.
####################################################