#Goodness-of-fit using test statistic based on PL (and GMM) for the skewed normal dist:

#Model: yi=b1*xi+b2*x^{2}_i+vi+ei where v_i~SN(0,sigma2,alpha) and ei~N(0,Di)
#where yi:sample survey of 4-person median income in yr 1989.
#xi:census 4-person median income in yr 1979. 

rm(list=ls(all=TRUE))

#---------------
# square-root-inverse of a matrix
mat.sqrt.inv<-function(A)
{
  ei<-eigen(A)
  d<-ei$values
  d<-(d+abs(d))/2
  d2<-1 / sqrt(d)
  d2[d == 0]<-0
  ans<-ei$vectors %*% diag(d2) %*% t(ei$vectors)
  return(ans)
}
#---------------
#Estimate beta and A:
ThetaNew.ran<-function(THETA, data)
{
  data=data.frame(xi, yib, m, p1,p,d, D)
  BETA=c(THETA[1:p1])
  A=THETA[p]
  
  ai<-rep(0,m)
  bi<-rep(0,m)
  ci<-rep(0,m)
  di<-rep(0,m)
  for(i in 1:m){
    Bi<-A/(A+D[i])
    ai[i]=(1-Bi)^2/(D[i]*(1+Bi))
    bi[i]=((1-Bi)^3*(3+Bi))/(2*D[i]^2*(1+Bi)^2) 
    ci[i]=((1-Bi)^2*(3+Bi))/(2*D[i]*(1+Bi)^2)
    di[i]<-((1-Bi)^2/(D[i]*(1+Bi)))*sqrt((2*A)/pi)
  }
  
  q=p+d  
  
  b.tem<-matrix(data=NA,m,q)
  
  for(i in 1:m){
    for(j in 1:p1){
      b.tem[i,j]<-ai[i]*xi[i]^(j)*(yib[i]-BETA[1]*xi[i]-BETA[2]*xi[i]^2)
    }  
    b.tem[i,p1+1]<-bi[i]*(yib[i]-BETA[1]*xi[i]-BETA[2]*xi[i]^2)^2-ci[i]
    b.tem[i,p1+2]<-di[i]*(yib[i]-BETA[1]*xi[i]-BETA[2]*xi[i]^2)
  }
  
  b<-apply(b.tem,2,sum)
  
  
  VAR.theta1<-array(data=NA, dim=c(q,q,m))
  for(i in 1:m){
    Bi<-A/(A+D[i])
    gi<-((1-Bi)^3)/(D[i]*(1+Bi)^2)
    hi<-gi*sqrt((2*A)/pi)
    COV.i<-matrix(data=NA,q,q)
    Gi<-gi*matrix(c(xi[i]^2,xi[i]^3,xi[i]^3,xi[i]^4),p1,p1,byrow=T)
    COV.i<-matrix(c(
                     Gi[1,1],Gi[1,2],0,(xi[i]*hi),
                     Gi[2,1],Gi[2,2],0,((xi[i]^2)*hi),
                     0,0,(2*(ci[i]^2)),0,
                     (xi[i]*hi),((xi[i]^2)*hi),0,(((2*A)/pi)*gi)),q,q,byrow=T)
    VAR.theta1[,,i]= COV.i 
  }
  
  VAR.theta<-apply(VAR.theta1,1:2,sum)
  B<-(mat.sqrt.inv(VAR.theta))%*%b
  
  
  b.theta1<-array(data=NA,dim=c(p,q,m))
  for(i in 1:m){
    Bi<-A/(A+D[i])
    Li<--ai[i]*(xi[i]^2)
    z1i<--2*bi[i]*xi[i]*(yib[i]-BETA[1]*xi[i])
    pi<-((1-Bi)*(3+Bi))/(2*D[i]*(1+Bi))
    pi.d.A<--(2*(1+Bi)^2+(1-Bi)*(3+Bi))/(2*(2*A+D[i])^2) 
    tem1<-(-2*bi[i])*pi+ai[i]*pi.d.A
    qi<-(3+Bi)/(2*(1+Bi))
    qi.d.A<--D[i]/((2*A+D[i])^2)
    tem2<-(-2*bi[i])*qi+ai[i]*qi.d.A
    z2i<-tem1*(yib[i]-BETA[1]*xi[i])^2-tem2
    tem3<-ai[i]/sqrt(2*A*pi)-(2*bi[i])*sqrt((2*A)/pi)
    z3i<-tem3*(yib[i]-BETA[1]*xi[i])
    b.theta.tem<-matrix(c(
      Li,z1i,-(xi[i]*di[i]),
      z1i,z2i,z3i),p,q,byrow=T)
    b.theta1[,,i]<- (1/m)*b.theta.tem   
  }
  
  b.theta<-apply(b.theta1, 1:2, sum)
  
  
  A1<-b.theta%*%(mat.sqrt.inv(VAR.theta))
  C<-A1%*%B
  Inv.C<--drop(1/m)*ginv(A1%*%t(A1))
  res.C<-(Inv.C)%*%C
  return(res.C)
} #function
#-----------------------
#Find test statistic B2:
FunB2.ran<-function(THETA, data)
{
  data=data.frame(xi, yib, m, p1,p,d, D)
  BETA=c(THETA[1:p1])
  A=THETA[p]
  
  ai<-rep(0,m)
  bi<-rep(0,m)
  ci<-rep(0,m)
  di<-rep(0,m)
  for(i in 1:m){
    Bi<-A/(A+D[i])
    ai[i]=(1-Bi)^2/(D[i]*(1+Bi))
    bi[i]=((1-Bi)^3*(3+Bi))/(2*D[i]^2*(1+Bi)^2) 
    ci[i]=((1-Bi)^2*(3+Bi))/(2*D[i]*(1+Bi)^2)
    di[i]<-((1-Bi)^2/(D[i]*(1+Bi)))*sqrt((2*A)/pi)
  }
  
  q=p+d  
  
  b.tem<-matrix(data=NA,m,q)
  
  for(i in 1:m){
    for(j in 1:p1){
      b.tem[i,j]<-ai[i]*xi[i]^(j)*(yib[i]-BETA[1]*xi[i]-BETA[2]*xi[i]^2)
    }  
    b.tem[i,p1+1]<-bi[i]*(yib[i]-BETA[1]*xi[i]-BETA[2]*xi[i]^2)^2-ci[i]
    b.tem[i,p1+2]<-di[i]*(yib[i]-BETA[1]*xi[i]-BETA[2]*xi[i]^2)
  }
  
  b<-apply(b.tem,2,sum)
  
  VAR.theta1<-array(data=NA, dim=c(q,q,m))
  for(i in 1:m){
    Bi<-A/(A+D[i])
    gi<-((1-Bi)^3)/(D[i]*(1+Bi)^2)
    hi<-gi*sqrt((2*A)/pi)
    COV.i<-matrix(data=NA,q,q)
    Gi<-gi*matrix(c(xi[i]^2,xi[i]^3,xi[i]^3,xi[i]^4),p1,p1,byrow=T)
    COV.i<-matrix(c(
      Gi[1,1],Gi[1,2],0,(xi[i]*hi),
      Gi[2,1],Gi[2,2],0,((xi[i]^2)*hi),
      0,0,(2*(ci[i]^2)),0,
      (xi[i]*hi),((xi[i]^2)*hi),0,(((2*A)/pi)*gi)),q,q,byrow=T)
    VAR.theta1[,,i]= COV.i 
  }
  
  VAR.theta<-apply(VAR.theta1,1:2,sum)
  B<-(mat.sqrt.inv(VAR.theta))%*%b
  
  B.2<-t(B)%*%B
  list(B.2=drop(B.2), b=b, VAR.theta=VAR.theta, B=B)
} #function
############################
library(sae)
library(nlme)
library(MASS)
library(psych)
library(sn)

m=51  
p1=2   #number of regression coefficents: beta1,beta2
p=3    #number of model parameters: beta1,beta2,A
d=1    #alpha parameter
q=p+d


Median<-read.table("fmedn89-1.txt",header=TRUE)
attach(Median)

yib<-rep(0,m)
D<-rep(0,m)
xi<-rep(0,m)

bb<-0
for(i in 1:m){
  yib[i]<-X4Med[13+bb]
  D[i]<-(X4_SE[13+bb])^2
  bb<-bb+13
}
yib
D
SD<-sqrt(D)

dd<-0
for(i in 1:m){xi[i]<-X4Med[2+dd]
dd<-dd+13}
xi

X<-cbind(xi,xi^2)
###################################
  PX<-X%*% solve(t(X)%*%X)%*%t(X)
  PXI<-diag(1,m)-PX
  D1<-diag(SD^2)
  #################Tailoring APPROACH######################
  
  #####We use the following estimtors (based on the PR and EBLUE) as initial values for 
  #the tailoring approach:
  
  A.hat.tem<-(t(yib)%*%PXI%*%yib-tr(PXI%*%D1))/(m-p1)
 A.hat.tem
  #  [1,] 18818686
  
  sumx.tem<-matrix(0,p1,p1)
  sumy.tem<-matrix(0,p1,1)
  for(i in 1:m){
    sumx.tem<-sumx.tem+( X[i,]%*%t(X[i,] ) ) /drop(A.hat.tem+D[i]) 
    sumy.tem<-sumy.tem+( X[i,]*yib[i] )/drop(A.hat.tem+D[i])
  }
  BETA.hat.tem<-solve(sumx.tem)%*%sumy.tem
   BETA.hat.tem
  #[,1]
  #xi  2.0686457882
  #-0.0000126125
############################################
  dat.ran=data.frame(xi=xi, yib=yib, m=m, p1=p1, p=p, d=d, D=D)
  
  old.par.ran<-c(BETA.hat.tem, A.hat.tem)
  dif.ran=rep(1,p)
  num.it<-0
  while ( (abs(dif.ran[1])>0.01 | abs(dif.ran[2])>0.01 | abs(dif.ran[3])>0.01) & num.it<100) {
    theta.hat.ran<-ThetaNew.ran(THETA=old.par.ran, data=dat.ran)
    new.par.ran<-old.par.ran+theta.hat.ran
    dif.ran=new.par.ran-old.par.ran
    old.par.ran<-new.par.ran
    num.it<-num.it+1
  }
  
  BETA.hat<-new.par.ran[1:p1]
  BETA.hat
  #[1]  2.0686457882 -0.0000126125
  
  A.hat<-new.par.ran[p]
  A.hat
  #[1] 18818686
  ######################Test statistic MAPLE##################
  B2.hat<-FunB2.ran(THETA=c(BETA.hat,A.hat), data=dat.ran)
  B.2.hat<-B2.hat$B.2
  B.2.hat
  #[1] 4.933703 
  #since 4.93>2.70 we reject the H0:vi~N(0,sigma2) at alpha=0.10. It means that vi has a skewed normal
  #distribution with skewness parameter $\alpha$ which can be estimated by the likelihood of the skewed normal model.
 ######################################################################################################################