#Goodness-of-fit using test statistic based on PL (and GMM) for the skewed normal dist:

#Model: yi=b1*xi+vi+ei where v_i~SN(0,A,alpha) and ei~N(0,Di)
#m=50

rm(list=ls(all=TRUE))

#---------------
# square-root-inverse of a matrix
mat.sqrt.inv<-function(A)
{
  ei<-eigen(A)
  d<-ei$values
  d<-(d+abs(d))/2
  d2<-1 / sqrt(d)
  d2[d == 0]<-0
  ans<-ei$vectors %*% diag(d2) %*% t(ei$vectors)
  return(ans)
}
#---------------
#Estimate beta and A:
ThetaNew.ran<-function(THETA, data)
{
  data=data.frame(xi, yib, m, p1,p,d, D)
  BETA=c(THETA[1:p1])
  A=THETA[p]
  
  ai<-rep(0,m)
  bi<-rep(0,m)
  ci<-rep(0,m)
  di<-rep(0,m)
  for(i in 1:m){
    Bi<-A/(A+D[i])
    ai[i]=(1-Bi)^2/(D[i]*(1+Bi))
    bi[i]=((1-Bi)^3*(3+Bi))/(2*D[i]^2*(1+Bi)^2) 
    ci[i]=((1-Bi)^2*(3+Bi))/(2*D[i]*(1+Bi)^2)
    di[i]<-((1-Bi)^2/(D[i]*(1+Bi)))*sqrt((2*A)/pi)
  }
  
  q=p+d  
  
  b.tem<-matrix(data=NA,m,q)
  
  for(i in 1:m){
    for(j in 1:p1){
      b.tem[i,j]<-ai[i]*xi[i]^(j)*(yib[i]-BETA[1]*xi[i])
    }  
    b.tem[i,p1+1]<-bi[i]*(yib[i]-BETA[1]*xi[i])^2-ci[i]
    b.tem[i,p1+2]<-di[i]*(yib[i]-BETA[1]*xi[i])
  }
  
  b<-apply(b.tem,2,sum)
  
  
  VAR.theta1<-array(data=NA, dim=c(q,q,m))
  for(i in 1:m){
    Bi<-A/(A+D[i])
    gi<-((1-Bi)^3)/(D[i]*(1+Bi)^2)
    hi<-gi*sqrt((2*A)/pi)
    COV.i<-matrix(data=NA,q,q)
    Gi<-gi*(xi[i]^2) 
    COV.i<-matrix(c(
                     Gi,0,(xi[i]*hi),
                     0,(2*(ci[i]^2)),0,
                     (xi[i]*hi),0,(((2*A)/pi)*gi)),q,q,byrow=T)
    VAR.theta1[,,i]= COV.i 
  }
  
  VAR.theta<-apply(VAR.theta1,1:2,sum)
  B<-(mat.sqrt.inv(VAR.theta))%*%b
  
  
  b.theta1<-array(data=NA,dim=c(p,q,m))
  for(i in 1:m){
    Bi<-A/(A+D[i])
    Li<--ai[i]*(xi[i]^2)
    z1i<--2*bi[i]*xi[i]*(yib[i]-BETA[1]*xi[i])
    pi<-((1-Bi)*(3+Bi))/(2*D[i]*(1+Bi))
    pi.d.A<--(2*(1+Bi)^2+(1-Bi)*(3+Bi))/(2*(2*A+D[i])^2) 
    tem1<-(-2*bi[i])*pi+ai[i]*pi.d.A
    qi<-(3+Bi)/(2*(1+Bi))
    qi.d.A<--D[i]/((2*A+D[i])^2)
    tem2<-(-2*bi[i])*qi+ai[i]*qi.d.A
    z2i<-tem1*(yib[i]-BETA[1]*xi[i])^2-tem2
    tem3<-ai[i]/sqrt(2*A*pi)-(2*bi[i])*sqrt((2*A)/pi)
    z3i<-tem3*(yib[i]-BETA[1]*xi[i])
    b.theta.tem<-matrix(c(
                             Li,z1i,-(xi[i]*di[i]),
                             z1i,z2i,z3i),p,q,byrow=T)
    b.theta1[,,i]<- (1/m)*b.theta.tem   
  }
  
  b.theta<-apply(b.theta1, 1:2, sum)
  
  A1<-b.theta%*%(mat.sqrt.inv(VAR.theta))
  C<-A1%*%B
  Inv.C<--drop(1/m)*ginv(A1%*%t(A1))
  res.C<-(Inv.C)%*%C
  return(res.C)
} #function
#-----------------------
#Find test statistic B2:
FunB2.ran<-function(THETA, data)
{
  data=data.frame(xi, yib, m, p1,p,d, D)
  BETA=c(THETA[1:p1])
  A=THETA[p]
  
  ai<-rep(0,m)
  bi<-rep(0,m)
  ci<-rep(0,m)
  di<-rep(0,m)
  for(i in 1:m){
    Bi<-A/(A+D[i])
    ai[i]=(1-Bi)^2/(D[i]*(1+Bi))
    bi[i]=((1-Bi)^3*(3+Bi))/(2*D[i]^2*(1+Bi)^2) 
    ci[i]=((1-Bi)^2*(3+Bi))/(2*D[i]*(1+Bi)^2)
    di[i]<-((1-Bi)^2/(D[i]*(1+Bi)))*sqrt((2*A)/pi)
  }
  
  q=p+d  
  
  b.tem<-matrix(data=NA,m,q)
  
  for(i in 1:m){
    for(j in 1:p1){
  b.tem[i,j]<-ai[i]*xi[i]^(j)*(yib[i]-BETA[1]*xi[i])
    }  
    b.tem[i,p1+1]<-bi[i]*(yib[i]-BETA[1]*xi[i])^2-ci[i]
    b.tem[i,p1+2]<-di[i]*(yib[i]-BETA[1]*xi[i])
  }
  
  b<-apply(b.tem,2,sum)
  
  
  VAR.theta1<-array(data=NA, dim=c(q,q,m))
  for(i in 1:m){
    Bi<-A/(A+D[i])
    gi<-((1-Bi)^3)/(D[i]*(1+Bi)^2)
    hi<-gi*sqrt((2*A)/pi)
    COV.i<-matrix(data=NA,q,q)
    Gi<-gi*(xi[i]^2)
    COV.i<-matrix(c(
      Gi,0,(xi[i]*hi),
      0,(2*(ci[i]^2)),0,
      (xi[i]*hi),0,(((2*A)/pi)*gi)),q,q,byrow=T)
    VAR.theta1[,,i]= COV.i 
  }
  
  VAR.theta<-apply(VAR.theta1,1:2,sum)
  B<-(mat.sqrt.inv(VAR.theta))%*%b
  
  B.2<-t(B)%*%B
  list(B.2=drop(B.2), b=b, VAR.theta=VAR.theta, B=B)
} #function

################################
runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(nlme)
library(MASS)
library(psych)
library(sn)

m=50
p1=1   #number of regression coefficents: beta1
p=2   #number of model parameters: beta1,A
d=1  #alpha parameter
q=p+d

n=m/2

Di.a=runif(n,3.5,4.5)
Di.b=runif(n,3.5,4.5)
D<-c(Di.a, Di.b)

SD<-sqrt(D)

beta1.a=1
beta1.b=1

A=10

xi<-runif(m,0,1)

R=5000

A.HAT<-rep(0,R)
BETA.HAT<-matrix(0,R,p1)
#V1.TM<-rep(0,R)
#V2.TM<-rep(0,R)
#Tm.HAT<-rep(0,R)
#Z.2<-rep(0,R)
#B.2<-rep(0,R)
B.2.hat<-rep(0,R)

BETA.HAT.PR<-matrix(0,R,p1)
A.HAT.PR<-rep(0,R)

NUM.IT<-rep(0,R)

#Z<-matrix(0,R,q)
#VarZ<-matrix(0,q,q)

#Z.hat<-matrix(0,R,q)
#VarZ.hat<-matrix(0,q,q)

#B1<-matrix(0,R,q)
#B.hat<-matrix(0,R,q)

for(r in 1:R){
  yib<-rep(0,m)
  for(i in 1:n){
     yib[i]<-rsn(1, beta1.a*xi[i], omega=sqrt(A+D[i]), alpha=0, tau=0,  dp=NULL)
  }
  for(i in (n+1):m){
       yib[i]<-rsn(1, beta1.b*xi[i], omega=sqrt(A+D[i]), alpha=0, tau=0,  dp=NULL)
  }
  X<-cbind(xi)
  ###################################
  PX<-X%*% solve(t(X)%*%X)%*%t(X)
  PXI<-diag(1,m)-PX
  D1<-diag(SD^2)
  #################Tailoring APPROACH######################
  #####We use the following estimtors (based on the PR and EBLUE) as initial values for 
  #the tailoring approach:
  
  A.hat.tem<-(t(yib)%*%PXI%*%yib-tr(PXI%*%D1))/(m-p1)
  A.HAT.PR[r]<-A.hat.tem  
  A.hat.tem<-ifelse(A.hat.tem<=0,0.0001,A.hat.tem)
  
  sumx.tem<-matrix(0,p1,p1)
  sumy.tem<-matrix(0,p1,1)
  for(i in 1:m){
    sumx.tem<-sumx.tem+( X[i,]%*%t(X[i,] ) ) /drop(A.hat.tem+D[i]) 
    sumy.tem<-sumy.tem+( X[i,]*yib[i] )/drop(A.hat.tem+D[i])
  }
  BETA.hat.tem<-solve(sumx.tem)%*%sumy.tem
  BETA.HAT.PR[r,]<-BETA.hat.tem
  ############################################
  dat.ran=data.frame(xi=xi, yib=yib, m=m, p1=p1, p=p, d=d, D=D)
  
  old.par.ran<-c(BETA.hat.tem, A.hat.tem)
  dif.ran=rep(1,p)
  num.it<-0
  while ( (abs(dif.ran[1])>0.01 | abs(dif.ran[2])>0.01 ) & num.it<100) {
    theta.hat.ran<-ThetaNew.ran(THETA=old.par.ran, data=dat.ran)
    new.par.ran<-old.par.ran+theta.hat.ran
    dif.ran=new.par.ran-old.par.ran
    old.par.ran<-new.par.ran
    num.it<-num.it+1
    # cat(num.it)
  }
  if (num.it==100){NUM.IT[r]<-1}
  
  BETA.HAT[r,]<-new.par.ran[1:p1]
  A.HAT[r]<-new.par.ran[p]
  A.hat<-A.HAT[r]  
  
  BETA.hat<-c(BETA.HAT[r,])
  A.hat<-ifelse(A.hat<=0, 0.00001, A.hat)
  A.hat<-ifelse(A.hat>=100, A, A.hat)
  ######################Test statistic MAPLE##################
  B2.hat<-FunB2.ran(THETA=c(BETA.hat,A.hat), data=dat.ran)
  B.2.hat[r]<-B2.hat$B.2
#  Z.hat[r,]<-B2.hat$b
#  VarZ.hat<-VarZ.hat+B2.hat$VAR.theta
#  B.hat[r,]<-B2.hat$B
  cat(r)
}#loop R

summary(A.HAT)
summary(BETA.HAT)

summary(A.HAT.PR)
summary(BETA.HAT.PR)

B.2.new.hat<-NULL

a<-1
for(r in 1:R){
  if(NUM.IT[r]<1){
    B.2.new.hat[a]<-B.2.hat[r]
    a<-a+1
  }
}
a-1
summary(B.2.new.hat)
######################################################################################################################
P.value.B2.hat.alpha0.01<-0
P.value.B2.hat.alpha0.05<-0
P.value.B2.hat.alpha0.1<-0
for(r in 1:(a-1)){
  if(B.2.new.hat[r]>6.63){P.value.B2.hat.alpha0.01<-P.value.B2.hat.alpha0.01+1} #d=1

  if(B.2.new.hat[r]>3.84){P.value.B2.hat.alpha0.05<-P.value.B2.hat.alpha0.05+1} #d=1

  if(B.2.new.hat[r]>2.70){P.value.B2.hat.alpha0.1<-P.value.B2.hat.alpha0.1+1} #d=1
}
P.value.B2.hat.alpha0.01/(a-1)
P.value.B2.hat.alpha0.05/(a-1)
P.value.B2.hat.alpha0.1/(a-1)
###############################################SIMULAITON STUDY (POWER)########################################################
#Model:yi=b1*xi+vi+ei where vi~SN(0,A,alpha)

#alpha=0.5

#Tm.HAT<-rep(0,R)
#Z.2<-rep(0,R)
#B.2<-rep(0,R)
B.2.hat<-rep(0,R)
             
for(r in 1:R){
  yib<-rep(0,m)
  for(i in 1:n){
    yib[i]<-rsn(1, beta1.a*xi[i], omega=sqrt(A+D[i]), alpha=0.5, tau=0,  dp=NULL)
  }
  for(i in (n+1):m){
    yib[i]<-rsn(1, beta1.b*xi[i], omega=sqrt(A+D[i]), alpha=0.5, tau=0,  dp=NULL)
  }
  #################Tailoring approach:######################
  A.hat<-A.HAT[r]
  A.hat<-ifelse(A.hat<=0,0.0001,A.hat)
  BETA.hat<-BETA.HAT[r,]
  #--------
  dat.ran=data.frame(xi=xi, yib=yib, m=m, p1=p1, p=p, d=d, D=D)
  ######################Test statistic PL##################
  B2hat<-FunB2.ran(THETA=c(BETA.hat,A.hat), data=dat.ran)
  B.2.hat[r]<-B2hat$B.2

    cat(r)
}#loop R

B.2.new<-NULL

a<-1
for(r in 1:R){
  if(NUM.IT[r]<1){
    B.2.new[a]<-B.2.hat[r]
    a<-a+1
  }
}
a-1
summary(B.2.new)
######################################################################################################################
Power.B2.alpha0.01<-0
Power.B2.alpha0.05<-0
Power.B2.alpha0.1<-0
for(r in 1:(a-1)){
    if(B.2.new[r]>6.63){Power.B2.alpha0.01<-Power.B2.alpha0.01+1}  #df=1

    if(B.2.new[r]>3.84){Power.B2.alpha0.05<-Power.B2.alpha0.05+1}  #df=1

    if(B.2.new[r]>2.70){Power.B2.alpha0.1<-Power.B2.alpha0.1+1}  #df=1
}
Power.B2.alpha0.01/(a-1)
Power.B2.alpha0.05/(a-1)
Power.B2.alpha0.1/(a-1)
########################################################################################################################
